# ai_hydra/constants/DSimCfg.py
#
#    AI Hydra
#    Author: Nadim-Daniel Ghaznavi
#    Copyright: (c) 2025-2026 Nadim-Daniel Ghaznavi
#    GitHub: https://github.com/NadimGhaznavi/ai_hydra
#    Website: https://ai-hydra.readthedocs.io/en/latest
#    License: GPL 3.0


from enum import StrEnum


class Phase(StrEnum):
    PRE_START = "pre_start"
    RUNTIME = "runtime"
